@@ -650,7 +650,7 @@ class DictTest(unittest.TestCase):
         self.assertTrue(_dict == _dict2)
 
         _values =  []
-        _values2 = ["Ahoj svete", "Hello world"] 
+        _values2 = ["Ahoj svete", "Hello world"]
         for v in _itervalues(_dict):
             _values.append(v)
         self.assertTrue(set(_values) == set(_values2))
@@ -671,7 +671,7 @@ class MDictTest(unittest.TestCase):
         wo1 = libcomps.Whiteout()
         wo2 = libcomps.Whiteout()
         self._test(wo1, wo2)
-    
+
     def _test(self, obj1, obj2):
         obj1["key"] = []
         self.assertTrue(obj1["key"] == libcomps.StrSeq())
@@ -718,24 +718,41 @@ class COMPSTest(unittest.TestCase):
         ret = comps2.fromxml_f(fname)
         self.assertTrue(ret == 0, comps2.get_last_errors())
 
-        compsdoc = comps2.xml_str()
-        compsdoc = compsdoc[0:-5] # make some error
         self.assertTrue(len(comps2.groups) == 3)
         self.assertTrue(len(comps2.categories) == 2)
         self.assertTrue(len(comps2.environments) == 0)
 
-        comps3 = libcomps.Comps()
-        self.assertRaises(libcomps.ParserError, comps3.fromxml_str, compsdoc)
-
-        self.assertTrue(len(comps3.groups) == 3)
-        self.assertTrue(len(comps3.categories) == 2)
-        self.assertTrue(len(comps3.environments) == 0)
         x = self.comps.xml_str(xml_options={})
         y = comps2.xml_str()
 
         self.assertTrue(x == y)
         os.remove(fname)
 
+        compsdoc = comps2.xml_str()
+        compsdoc = compsdoc[0:-5] # make some error
+        comps3 = libcomps.Comps()
+        self.assertRaises(libcomps.ParserError, comps3.fromxml_str, compsdoc)
+
+        INVALID_COMPS_XML = "invalid xml"
+        comps4 = libcomps.Comps()
+        self.assertRaises(libcomps.ParserError, comps4.fromxml_str, str(INVALID_COMPS_XML))
+
+        VALID_XML_NO_COMPS = "<whatever />"
+        comps5 = libcomps.Comps()
+        ret = comps5.fromxml_str(str(VALID_XML_NO_COMPS))
+        # return code 1 is non fatal error
+        self.assertTrue(ret == 1, comps5.get_last_errors())
+
+        VALID_XML_HEADER_NO_COMPS = """<?xml version="1.0" encoding="UTF-8"?>
+        <!DOCTYPE variants PUBLIC "-//Red Hat, Inc.//DTD Variants info//EN" "variants.dtd">
+        <variants>
+        </variants>"""
+        comps6 = libcomps.Comps()
+        ret = comps6.fromxml_str(str(VALID_XML_HEADER_NO_COMPS))
+        # return code 1 is non fatal error
+        self.assertTrue(ret == 1, comps6.get_last_errors())
+
+
     #@unittest.skip("")
     def test_fedora(self):
         comps = libcomps.Comps()
@@ -961,9 +978,9 @@ class COMPSTest(unittest.TestCase):
         comps = libcomps.Comps()
         ret = comps.fromxml_f("comps/f21-rawhide-comps.xml")
         env = comps.environments[0]
-        self.assertEqual(env.name_by_lang['cs'], u'Pracovní prostředí GNOME')
+        self.assertEqual(env.name_by_lang['cs'], 'Pracovní prostředí GNOME')
         self.assertEqual(env.desc_by_lang['de'],
-                         u'GNOME ist eine hoch-intuitive und benutzerfreundliche Benutzeroberfläche')
+                         'GNOME ist eine hoch-intuitive und benutzerfreundliche Benutzeroberfläche')
         group_ids = ("base-x", "standard", "core", "dial-up",
                      "fonts", "input-methods", "multimedia",
                      "hardware-support", "printing", "firefox",
@@ -974,7 +991,7 @@ class COMPSTest(unittest.TestCase):
         elif hasattr(self, "assertItemsEqual"):
             _f = self.assertSequenceEqual
         else:
-           _f = utest.assertSequenceEqual
+            _f = utest.assertSequenceEqual
 
         _f([id_.name for id_ in env.group_ids],
                               group_ids)
@@ -994,8 +1011,8 @@ class COMPSTest(unittest.TestCase):
         env = comps.categories[0].name_by_lang.clear()
         env = comps.groups[0].name_by_lang.clear()
 
-        self.assertEqual(str(comps.categories[0].name_by_lang), u'{}')
-        self.assertEqual(str(comps.groups[0].name_by_lang), u'{}')
+        self.assertEqual(str(comps.categories[0].name_by_lang), '{}')
+        self.assertEqual(str(comps.groups[0].name_by_lang), '{}')
 
     #@unittest.skip("")
     def test_xml_options(self):
@@ -1254,7 +1271,7 @@ if __name__ == "__main__":
     #unittest.main()
     #suite = unittest.TestLoader().loadTestsFromTestCase(Category_Test)
     #ret = MyRunner(verbosity=2).run(suite)
-    
+
     #unittest.TextTestRunner(verbosity=2).run(suite)
     #suite = unittest.TestLoader().loadTestsFromTestCase(GroupTest)
     #ret = MyRunner(verbosity=2).run(suite)
@@ -1268,6 +1285,3 @@ if __name__ == "__main__":
     #MyRunner(verbosity=2).run(suite)
     #suite = unittest.TestLoader().loadTestsFromTestCase(EnvListTest)
     #MyRunner(verbosity=2).run(suite)
-
-
-
